local inspect = require 'inspect'
local actions = {}

-- keep <script>...</script> blocks intact
local function discount(str)
	return ""
end

local function filter(t, p)
	local r = {}
	for _,v in ipairs(t) do
		if p(v) then r[#r+1] = v end
	end
	return r
end

local function anchorname(s)
	return s:gsub('%s$', ''):gsub('%s','_')
end

-- generic
function actions.textblock(info)
	return info[1]
end

function actions.section(info)
	return table.concat{
		'\n\n',
		info[1], '\n',
		string.rep(({"-", "^", '"'})[info[3]], #info[1]),
		info[4]
	}
end

-- function
function actions.parameters(info)
	return info[1]:gsub("\n=([^=]+)=", function(m,...) return ("\n   :param %s: "):format(m) end):gsub("\n\t", " ")
end

function actions.returns(info)
	return info[1]:gsub("\n=([^=]+)=", function(m,...) return ("\n   :returns: ``%s`` "):format(m) end):gsub("\n\t", " ")
end

function actions.example(info)
	return ("\n**Example**::%s"):format(info[1])
end

function actions.sketch(info)
	return ("\n**Sketch**::%s"):format(info[1])
end

actions['function'] = function(info)
	local arg_delim = info.has_table_args and {'{','}'} or {'(',')'}
	local out = {
		'.. function:: ', info[1], arg_delim[1], info[2], arg_delim[2]
	}

	-- parameter list
	local parameters = filter(info[5], function(v) return v.type == 'parameters' end)
	for _, r in ipairs(parameters) do
		out[#out+1] = actions.parameters(r)
	end

	-- returns list
	local returns = filter(info[5], function(v) return v.type == 'returns' end)
	for _, r in ipairs(returns) do
		out[#out+1] = actions.returns(r)
	end

	-- description
	out[#out+1] = info[4]:gsub('`', '``')

	-- examples
	local examples = filter(info[5], function(v) return v.type == 'example' end)
	for _,s in ipairs(examples) do
		out[#out+1] = actions.example(s)
	end

	-- sketch
	local sketch = filter(info[5], function(v) return v.type == 'sketch' end)
	for _,s in ipairs(sketch) do
		out[#out+1] = actions.sketch(s)
	end

	out[#out+1] = '\n'
	return table.concat(out)
end

-- module
function actions.module(info)
	local modname = info[1]
	local out = {
		modname, '\n',
		("="):rep(#modname), '\n', info[3]
	}

	-- create detailed reference
	for _,info in ipairs(info[4]) do
		local s = actions[info.type](info)
		out[#out+1] = s:gsub('{{MODULE}}', modname)
	end

	return table.concat(out)
end


function actions.title(info)
	return ""
end

function actions.preprocess(doctree)
	return doctree
end

function actions.postprocess(out)
	return out
end

return actions
